#!/bin/bash

# -----------------------------------------------------------------------------
# Launches an Axon Ivy Engine.
#
# Copyright 1994 - 2022 Axon Ivy AG
# -----------------------------------------------------------------------------
function useSmallMemoryJavaOptionsIfNotSet()
{
  if [ -z ${JAVA_OPTS_IVY+x} ]; then
    JAVA_OPTS_IVY="-XX:-OmitStackTraceInFastThrow -Xmx256m"
    export JAVA_OPTS_IVY
  fi  
}
useSmallMemoryJavaOptionsIfNotSet

# resolve links - $0 may be a softlink
PRG="$0"

while [ -h "$PRG" ]; do
  ls=`ls -ld "$PRG"`
  link=`expr "$ls" : '.*-> \(.*\)$'`
  if expr "$link" : '.*/.*' > /dev/null; then
    PRG="$link"
  else
    PRG=`dirname "$PRG"`/"$link"
  fi
done

# Get standard environment variables
PRGDIR=`dirname "$PRG"`

# change to parent dir
cd "$PRGDIR/.."

# set current directory as AxonIvyHome
AxonIvyHome=`pwd`

# Build classpath
# add all jars in lib/boot to classpath
for jar in "$AxonIvyHome/system/lib/boot/"*.jar;
do
  CLASSPATH="$CLASSPATH":"$jar"
done

# Set standard commands for invoking Java.
if [ -x "$AxonIvyHome/jre/bin/java" ]; then
    JAVA="$AxonIvyHome/jre/bin/java"
elif [ -x "$IVY_JAVA_HOME/bin/java" ]; then
    JAVA="$IVY_JAVA_HOME/bin/java"
elif [ -x "$JAVA_HOME/bin/java" ]; then
    JAVA="$JAVA_HOME/bin/java"
else
    JAVA=`which java`
fi

# Check Java binary exists
if [ ! -x "$JAVA" ]; then
    echo "Could not find any executable java binary. Please install java in your PATH or set JAVA_HOME"
    exit 1
fi

# Check JVM Version
jdk_version() {
  local result  
  local IFS=$'\n'
  # remove \r for Cygwin
  local lines=$("$JAVA" -Xms32M -Xmx32M -version 2>&1 | tr '\r' '\n')
  if [[ -z $JAVA ]]
  then
    result=no_java
  else
    for line in $lines; do
      if [[ (-z $result) && ($line = *"version \""*) ]]
      then
        local ver=$(echo $line | sed -e 's/.*version "\(.*\)"\(.*\)/\1/; 1q')
        # on macOS, sed doesn't support '?'
        if [[ $ver = "1."* ]]
        then
          result=$(echo $ver | sed -e 's/1\.\([0-9]*\)\(.*\)/\1/; 1q')
        else
          result=$(echo $ver | sed -e 's/\([0-9]*\)\(.*\)/\1/; 1q')
        fi
      fi
    done
  fi
  echo "$result"
}
VERSION="$(jdk_version)"
if [ "${VERSION}" -lt "17" ]; then
    echo "JVM must have version 17 or greater ($JAVA), but has version $VERSION."
    exit 1;
fi

# Check sudo
if [ "$USER" = "root" ]; then
     echo "Cannot start the engine with user 'root'. Please run it with a normal user such as 'ivy'."
     exit 1;
fi

# Set locale to de_DE@euro so that java file.encoding is set to ISO-8856-15. 
# Uncomment this line if file encoding is wrong when starting Axon Ivy Engine 
# in a runlevel.
#
# export LC_CTYPE=de_DE@euro

# start engine
if [ "x$1" = 'xstartdaemon' ] || [ "x$1" = 'x-startdaemon' ]; then
    nohup "$JAVA" -server -cp "$CLASSPATH" -Dosgi.install.area="$AxonIvyHome/system" -Dosgi.instance.area="$AxonIvyHome/system/workspace" $JAVA_OPTS_IVY org.eclipse.equinox.launcher.Main "$@" & echo $! > bin/AxonIvyEngine.pid
    consoleLog="$AxonIvyHome/logs/console.log"
    touch "$consoleLog"
    # wait until the 'running' log line appears / or the engine PID ends!
    tail -f -n0 --pid=$! "$consoleLog" | while read LOGLINE
    do
      [[ "${LOGLINE}" == *"Axon Ivy Engine is running"* ]] && pkill -P $$ tail && exit 0
    done
else
    exec "$JAVA" -server -cp "$CLASSPATH" $JAVA_OPTS_IVY_SYSTEM -Dosgi.install.area="$AxonIvyHome/system" -Dosgi.instance.area="$AxonIvyHome/system/workspace" $JAVA_OPTS_IVY org.eclipse.equinox.launcher.Main "$@"
fi
